@@ -4,10 +4,15 @@ from typing import List, Optional
 import typer
 from loguru import logger
 
-from tracarbon import CarbonEmission, Country, EnergyConsumption
 from tracarbon.builder import TracarbonBuilder
-from tracarbon.exporters import Exporter, Metric, Tag
-from tracarbon.hardwares import HardwareInfo
+from tracarbon.exporters import Exporter, MetricGenerator
+from tracarbon.general_metrics import (
+    CarbonEmissionGenerator,
+    CarbonEmissionKubernetesGenerator,
+    EnergyConsumptionGenerator,
+    EnergyConsumptionKubernetesGenerator,
+)
+from tracarbon.locations import Country
 
 app = typer.Typer()
 
@@ -25,14 +30,14 @@ def list_exporters(displayed: bool = True) -> List[str]:
 
 def get_exporter(
     exporter_name: str,
-    metrics: List[Metric],
+    metric_generators: List[MetricGenerator],
     tracarbon_builder: TracarbonBuilder = TracarbonBuilder(),
 ) -> Exporter:
     """
     Get the exporter based on the name with its metrics.
 
     :param exporter_name: the name of the exporter
-    :param metrics: the list of the associated metrics
+    :param metric_generators: the list of the metrics generators
     :param tracarbon_builder: the configuration of Tracarbon
     :return: the configured exporter
     """
@@ -47,13 +52,14 @@ def get_exporter(
     except Exception as exception:
         logger.exception("This exporter initiation failed.")
         raise exception
-    return selected_exporter(metrics=metrics, metric_prefix_name=tracarbon_builder.configuration.metric_prefix_name)  # type: ignore
+    return selected_exporter(metric_generators=metric_generators, metric_prefix_name=tracarbon_builder.configuration.metric_prefix_name)  # type: ignore
 
 
 def run_metrics(
     exporter_name: str,
     country_code_alpha_iso_2: Optional[str] = None,
     running: bool = True,
+    containers: bool = False,
 ) -> None:
     """
     Run the metrics with the selected exporter
@@ -61,6 +67,7 @@ def run_metrics(
     :param country_code_alpha_iso_2: the alpha iso2 country name where it's running
     :param running: keep running the metrics
     :param exporter_name: the exporter name to run
+    :param containers: activate the containers feature
     :return:
     """
     tracarbon_builder = TracarbonBuilder()
@@ -68,56 +75,23 @@ def run_metrics(
         co2signal_api_key=tracarbon_builder.configuration.co2signal_api_key,
         country_code_alpha_iso_2=country_code_alpha_iso_2,
     )
-    platform = HardwareInfo.get_platform()
-    metrics = list()
-    metrics.append(
-        Metric(
-            name="energy_consumption",
-            value=EnergyConsumption.from_platform().run,
-            tags=[
-                Tag(key="platform", value=platform),
-                Tag(key="location", value=location.name),
-            ],
+    metric_generators: List[MetricGenerator] = [
+        EnergyConsumptionGenerator(location=location),
+        CarbonEmissionGenerator(
+            location=location,
+        ),
+    ]
+    if containers:
+        metric_generators.extend(
+            [
+                EnergyConsumptionKubernetesGenerator(location=location),
+                CarbonEmissionKubernetesGenerator(location=location),
+            ]
         )
-    )
-    metrics.append(
-        Metric(
-            name="co2_emission",
-            value=CarbonEmission(
-                co2signal_api_key=tracarbon_builder.configuration.co2signal_api_key,
-                location=location,
-            ).run,
-            tags=[
-                Tag(key="platform", value=platform),
-                Tag(key="location", value=location.name),
-                Tag(key="source", value=location.co2g_kwh_source.value),
-            ],
-        )
-    )
-    metrics.append(
-        Metric(
-            name="hardware_memory_usage",
-            value=HardwareInfo().get_memory_usage,
-            tags=[
-                Tag(key="platform", value=platform),
-                Tag(key="location", value=location.name),
-            ],
-        )
-    )
-    metrics.append(
-        Metric(
-            name="hardware_cpu_usage",
-            value=HardwareInfo().get_cpu_usage,
-            tags=[
-                Tag(key="platform", value=platform),
-                Tag(key="location", value=location.name),
-            ],
-        )
-    )
     try:
         exporter = get_exporter(
             exporter_name=exporter_name,
-            metrics=metrics,
+            metric_generators=metric_generators,
             tracarbon_builder=tracarbon_builder,
         )
         tracarbon = tracarbon_builder.build(
@@ -136,13 +110,17 @@ def run_metrics(
 
 @app.command()
 def run(
-    exporter_name: str = "Stdout", country_code_alpha_iso_2: Optional[str] = None
+    exporter_name: str = "Stdout",
+    country_code_alpha_iso_2: Optional[str] = None,
+    containers: bool = False,
 ) -> None:
     """
     Run Tracarbon.
     """
     run_metrics(
-        exporter_name=exporter_name, country_code_alpha_iso_2=country_code_alpha_iso_2
+        exporter_name=exporter_name,
+        country_code_alpha_iso_2=country_code_alpha_iso_2,
+        containers=containers,
     )
 
 
