@@ -1,5 +1,5 @@
 from tracarbon.exporters.datadog_exporter import *
 from tracarbon.exporters.exporter import *
 from tracarbon.exporters.json_exporter import *
-from tracarbon.exporters.promotheus_exporter import *
+from tracarbon.exporters.prometheus_exporter import *
 from tracarbon.exporters.stdout import *
