@@ -1,16 +1,18 @@
 import asyncio
 from abc import ABCMeta, abstractmethod
 from threading import Event, Timer
-from typing import Awaitable, Callable, List, Optional
+from typing import AsyncGenerator, Awaitable, Callable, List, Optional
 
 from loguru import logger
 from pydantic import BaseModel
 
+from tracarbon.hardwares.hardware import HardwareInfo
+
 
 class Exporter(BaseModel, metaclass=ABCMeta):
     """The Exporter interface."""
 
-    metrics: List["Metric"]
+    metric_generators: List["MetricGenerator"]
     event: Optional[Event] = None
     stopped: bool = False
     metric_prefix_name: Optional[str] = None
@@ -21,12 +23,11 @@ class Exporter(BaseModel, metaclass=ABCMeta):
         arbitrary_types_allowed = True
 
     @abstractmethod
-    async def launch(self, metric: "Metric") -> None:
+    async def launch(self, metric_generator: "MetricGenerator") -> None:
         """
         Launch the exporter.
 
-        :param metric: the metric to send
-        :return:
+        :param metric_generator: the metric generator
         """
         pass
 
@@ -34,7 +35,7 @@ class Exporter(BaseModel, metaclass=ABCMeta):
         """
         Start the exporter and a dedicated timer configured with the configured timeout.
 
-        :return:
+        :param: interval_in_seconds: the interval for the timer
         """
         self.stopped = False
         if not self.event:
@@ -61,15 +62,14 @@ class Exporter(BaseModel, metaclass=ABCMeta):
 
     async def _launch_all(self) -> None:
         """
-        Launch the exporter with all the metrics.
-
-        :return:
+        Launch the exporter with all the metric generators.
         """
-        for metric in self.metrics:
-            logger.debug(f"Running Metric[{metric}].")
-            await self.launch(metric=metric)
+        for metric_generator in self.metric_generators:
+            logger.debug(f"Running MetricGenerator[{metric_generator}].")
+            await self.launch(metric_generator=metric_generator)
 
     @classmethod
+    @abstractmethod
     def get_name(cls) -> str:
         """
         Get the name of the exporter.
@@ -117,3 +117,19 @@ class Metric(BaseModel):
         :param separator: the separator to insert between the key and value.
         """
         return [f"{tag.key}{separator}{tag.value}" for tag in self.tags]
+
+
+class MetricGenerator(BaseModel):
+    """
+    MetricGenerator generates metrics for the Exporter.
+    """
+
+    metrics: List[Metric]
+    platform: str = HardwareInfo.get_platform()
+
+    async def generate(self) -> AsyncGenerator[Metric, None]:
+        """
+        Generate a metric.
+        """
+        for metric in self.metrics:
+            yield metric
